// Util.cpp

#include "stdafx.h"
#include "Util.h"
#include "Version.h"
#include "Constants.h"

namespace WaveSoftware
{

   bool g_DebugPlatformEnabled = false;
   std::string g_DebugPlatform = ""; // DONT'T assign CSTR_EMPTY_STRING

   bool Util::GetFromMap(const StringMap& data, const std::string& key, std::string& val)
   {
      StringMap::const_iterator i;
      StringMap::const_iterator j = data.end();

      i = data.find(key);

      if(i == j)
         return(false);

      val = (*i).second;

      return(true);
   }

   bool Util::GetFromMap(const StringMap& data, const std::string& key, int& val)
   {
      std::string tmp;

      if(!(GetFromMap(data, key, tmp)))
         return(false);

      return(GetFromString(tmp, val));
   }

   bool Util::GetFromMap(const StringMap& data, const std::string& key, bool& val)
   {
      std::string tmp;

      if(!(GetFromMap(data, key, tmp)))
         return(false);

      return(GetFromString(tmp, val));
   }

   bool Util::GetFromString(const std::string& str, int& val)
   {
      val = atoi(str.c_str());

      return(true);
   }

   bool Util::GetFromString(const std::string& str, bool& val)
   {
      // Legal values for str are:
      //
      // true, yes, 1
      // false, no, 0
      //
      // The textual representations can be mixed case.

      std::string tmp = str;
      std::string::iterator i = tmp.begin();
      std::string::iterator j = tmp.end();

      for(; i != j; i++)
         *i = tolower(*i);

      bool found = false;
      val = false;

      if((0 == tmp.compare(CSTR_TRUE)) ||
         (0 == tmp.compare(CSTR_YES)) ||
         (0 == tmp.compare(CSTR_ONE)))
      {
         val = true;
         found = true;
      }
      else if((0 == tmp.compare(CSTR_FALSE)) ||
              (0 == tmp.compare(CSTR_NO)) ||
              (0 == tmp.compare(CSTR_ZERO)))
      {
         val = false;
         found = true;
      }

      return(found);
   }

   bool Util::GetFromString(const std::string& str, Versions& versions)
   {
      // str should be a comma separated list of version range
      // specifications:
      //
      // 5.0.0.1-5.5.0.0,6.0.540.2-6.1.2830.10
      //
      // Each version in the string is assingned to an instanc of
      // Version.  All Version instances are stored in the supplied
      // Versions container.

      TCHAR* buffer = new TCHAR[str.length() + 1];
      _tcscpy(buffer, str.c_str());

      TCHAR* token = _tcstok(buffer, CSTR_COMMA.c_str());

      while(NULL != token)
      {
         TCHAR* delim = _tcschr(token, '-');
         TCHAR* src = token;
         TCHAR* dst = delim + 1;

         *delim = 0;

         Version* srcver = new Version(src);
         Version* dstver = new Version(dst);

         // Check that the versions are OK.
         if((!(srcver->IsValid()) || !(dstver->IsValid())) ||
            (srcver->Compare(*dstver) > 0))
         {
            delete srcver;  srcver = NULL;
            delete dstver;  dstver = NULL;

            // We also need to delete the string buffer we created.
            delete [] buffer;  buffer = NULL;

            return(false);
         }

         versions.push_back(srcver);
         versions.push_back(dstver);

         token = _tcstok(NULL, CSTR_COMMA.c_str());
      }

      return(true);
   }

   void Util::SplitPath(const std::string& path, std::string& drive, std::string& dir, std::string& fname, std::string& ext)
   {
      char cdrive[_MAX_DRIVE];
      char cdir[_MAX_DIR];
      char cfname[_MAX_FNAME];
      char cext[_MAX_EXT];

      _splitpath(path.c_str(), cdrive, cdir, cfname, cext);

      drive = cdrive;
      dir   = cdir;
      fname = cfname;
      ext   = cext;
   }

   bool Util::ShellExecute(const TCHAR* cmd, const TCHAR* params)
   {
      HINSTANCE inst = NULL;

      inst = ::ShellExecute((HWND) NULL,
                            CSTR_OPEN_COMMAND.c_str(),
                            cmd,
                            params,
                            CSTR_EMPTY_STRING.c_str(),
                            SW_SHOW);

      return((int) inst > 32);
   }

   const std::string& Util::GetCurrentPlatform()
   {
      if(g_DebugPlatformEnabled)
         return(g_DebugPlatform);

      OSVERSIONINFO osi;
      BOOL r = FALSE;

      ::ZeroMemory(&osi, sizeof(OSVERSIONINFO));
      osi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

      r = ::GetVersionEx(&osi);

      if(r)
      {
         switch(osi.dwMajorVersion)
         {
         case 3:
            if(51 == osi.dwMinorVersion)
               return(CSTR_NT351);
            else
               return(CSTR_UNKNOWN_PLATFORM);

         case 4:
            switch(osi.dwMinorVersion)
            {
            case 0:
               // Could be W95 or NT4
               if(VER_PLATFORM_WIN32_WINDOWS == osi.dwPlatformId)
                  return(CSTR_W95);
               else if(VER_PLATFORM_WIN32_NT == osi.dwPlatformId)
                  return(CSTR_NT4);
               else
                  return(CSTR_UNKNOWN_PLATFORM);

            case 10:
               return(CSTR_W98);

            case 90:
               return(CSTR_WME);

            default:
               return(CSTR_UNKNOWN_PLATFORM);
            }

         case 5:
            switch(osi.dwMinorVersion)
            {
            case 0:
               return(CSTR_W2K);

            case 1:
               return(CSTR_WXP);

            default:
               return(CSTR_UNKNOWN_PLATFORM);
            }

         default:
            return(CSTR_UNKNOWN_PLATFORM);
         }
      }

      return(CSTR_UNKNOWN_PLATFORM);
   }

   const std::string& Util::GetCurrentPlatformString()
   {
      const std::string& cp = GetCurrentPlatform();

      if(cp == CSTR_W95) return(CSTR_WINDOWS_95);
      if(cp == CSTR_W98) return(CSTR_WINDOWS_98);
      if(cp == CSTR_WME) return(CSTR_WINDOWS_ME);
      if(cp == CSTR_NT351) return(CSTR_WINDOWS_NT_351);
      if(cp == CSTR_NT4) return(CSTR_WINDOWS_NT_4);
      if(cp == CSTR_W2K) return(CSTR_WINDOWS_2000);
      if(cp == CSTR_WXP) return(CSTR_WINDOWS_XP);

      return(CSTR_UNKNOWN_PLATFORM);
//      return(CSTR_ZERO);
   }

   int Util::CompareVersionWithVersions(const Version& version, const Versions& versions)
   {
      // Return values:
      //
      // 1 - version falls into one of the version ranges.
      //
      // 3 - version is greater than the last of the versions.
      //
      // 4 - everything else

      // Do the simple boundary tests first.
      Version* v = versions[0];

      if(v->Compare(version) > 0)
         return(4);

      v = versions[versions.size() - 1];

      if(v->Compare(version) < 0)
         return(3);

      // Now see if the version lies in one of the ranges.
      int i = 0;
      int j = versions.size() - 1;

      while(i < j)
      {
         Version* low = versions[i];
         Version* hgh = versions[i + 1];

         i += 2;

         int dlow = low->Compare(version);
         int dhgh = hgh->Compare(version);

         if((dlow <= 0) && (dhgh >= 0))
            return(1);
      }

      return(4);
   }

   void Util::DestroyVersions(Versions& versions)
   {
      Versions::iterator i = versions.begin();
      Versions::iterator j = versions.end();

      for(; i != j; i++)
      {
         Version* v = *i;

         delete v;  v = NULL;
      }

      versions.clear();
   }

} // end of namespace

